﻿//'*****************************************************************
// *                                                               *
// *       Copyright (c) Fast Corporation, 2007                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************/

// -------------------------------------------------------------------
// FVX Sample Program
// メインダイアログ
//
// 作成者 : R.Nagashima
//
// 履歴 : 2007/03/09 初版
// -------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Threading;

//FVXデータ構造の使用宣言
using FVDATASLib;

namespace Hough
{
	public partial class frmMain : Form
	{
		public frmMain()
		{
			InitializeComponent();
		}

		//処理範囲
		FvRectangleClass m_ExeRegion = new FvRectangleClass();
		// ビデオデバイスがオープンされているか
		private bool m_blnVideoOpened = false;
		// ビデオ設定ファイル名（フルパス）
		private string m_strVideoConfFilePath = "";
		// 初期ディレクトリ
		private string m_strDirectory;

		private int m_intErrR, m_intErrQ;

		private void frmMain_Load(object sender, EventArgs e)
		{
			int IntErr;
			frmSelectvideoConfFile fVideoConf = new frmSelectvideoConfFile();
			frmLicense fLicense = new frmLicense();
			

			// 変数初期化
			m_blnVideoOpened = false;
			btnGrab.Enabled = false;

			//画面の生成
			axFvxDisplayEx1.CreateDisplay();

			//処理範囲の初期値設定(画像の全面に設定）
			m_ExeRegion.xs = -1;
			m_ExeRegion.xe = -1;
			m_ExeRegion.ys = -1;
			m_ExeRegion.ye = -1;

			//作業ディレクトリ退避
			m_strDirectory = Directory.GetCurrentDirectory();

			//使用許諾の確認
			if (fLicense.ShowDialog() == DialogResult.OK)
			{
				//ビデオ設定ファイルの選択
				if (fVideoConf.ShowDialog() == DialogResult.OK)
					m_strVideoConfFilePath = fVideoConf.strVideoConfFilePath;

				if (fVideoConf.chkNotUseCamera.Checked == false)
				{
					//ビデオ設定ファイルをコンポーネントに設定
					axFvxVideo1.VideoConfFile = m_strVideoConfFilePath;

					//デバイスオープン
					try
					{
						axFvxVideo1.OpenDevice();
					}
					catch (Exception)
					{
						axFvxVideo1.GetErrReport(out IntErr);
						MessageBox.Show("ビデオデバイスのオープンに失敗しました｡ ErrCode= " + IntErr.ToString() + "\r\n画像ファイルのみの利用になります。");
						return;
					}

					//VideoConfFileの変更後、OpenDeviceを行うと、トリガーモードの変更が伴います
					//このため、弊社キャプチャボードFVC02では350msのセットアップタイムが必要になります｡
					//本プログラムでは簡略化のために、一律に350msのセットアップタイムを確保していますが、
					//お使いのキャプチャボードに合わせて変更してください。
					Thread.Sleep(350);

					//GrabSync/GrabASyncメソッドではImegeサイズの変更を行いませんので、
					//あらかじめChangeImageSizeメソッドにてImageのサイズを変更してください
					//（取り込みサイズとImageのサイズが違う場合エラーになります）
					axFvxVideo1.ChangeImageSize(axFvxImage1.GetOcx());
					try
					{
						// 同期取込
						axFvxVideo1.GrabSync(axFvxImage1.GetOcx());
					}
					catch (Exception)
					{
						axFvxVideo1.GetErrReport(out IntErr);
						MessageBox.Show("ビデオの取込に失敗しました｡ ErrCode= " + IntErr.ToString() + "\r\n画像ファイルのみの利用になります。");
						return;
					}

					//ビデオデバイスの状態を保存
					m_blnVideoOpened = true;
					btnGrab.Enabled = true;

					//画像を表示
					axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
				}
			}
			else
			{
				//終了
				Application.Exit();
				return;
			}

		}

		//終了時の処理
		private void frmMain_FormClosed(object sender, FormClosedEventArgs e)
		{
			if (m_blnVideoOpened)
			{
				// ビデオデバイスがオープンされていた場合はクローズします｡
				axFvxVideo1.CloseDevice();
			}
		}

		//カメラ画像取り込み
		private void btnGrab_Click(object sender, EventArgs e)
		{
			//画像ファイル読み込みによって、画像サイズが初期化時から
			//変更されている可能性があるため、ChangeImageSizeを行います。
			//画像サイズが変更される可能性が無い場合は、初期化時に一度だけ行えば問題有りません。
			axFvxVideo1.ChangeImageSize(axFvxImage1.GetOcx());

			try
			{
				//同期取込
				axFvxVideo1.GrabSync(axFvxImage1.GetOcx());
			}
			catch (Exception)
			{
				int IntErr;
				axFvxVideo1.GetErrReport(out IntErr);
				MessageBox.Show("ビデオの取込に失敗しました｡ ErrCode= " + IntErr.ToString());
			}

			//取り込んだ画像を画面に表示
			axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());

		}

		//画像ファイル読込
		private void btnLoadImage_Click(object sender, EventArgs e)
		{
			// ダイアログにて画像ファイルを選択します｡
			openFileDialog1.InitialDirectory = m_strDirectory;
			openFileDialog1.Filter = "bitmap file(*.bmp)|*.bmp";
			openFileDialog1.DefaultExt = "*.bmp";
			openFileDialog1.ShowDialog();

			if (openFileDialog1.FileName.Length > 0)
			{
				//FileAccessコンポーネントを用いて、
				//ファイルからImageに画像を読み込みます｡
				axFvxFileAccess1.FilePath = openFileDialog1.FileName;

				try
				{
					axFvxFileAccess1.LoadBitmap(axFvxImage1.GetOcx());
				}
				catch (Exception)
				{
					int IntErr = 0;
					axFvxFileAccess1.GetErrReport(out IntErr);
					MessageBox.Show("読込みに失敗しました｡ ErrCode= " + IntErr.ToString());
					return;
				}

				//画像タイプチェック
				if ((axFvxImage1.BitPerPixel != 8) || (axFvxImage1.ImageType != FVXCOMIMAGELib.FVXIMAGE_IMAGE_TYPE.imageGray))
				{
					MessageBox.Show("8bitグレイの画像ではありません｡");
					axFvxImage1.BitPerPixel = 8;
					axFvxImage1.ImageType = FVXCOMIMAGELib.FVXIMAGE_IMAGE_TYPE.imageGray;
					axFvxImage1.ImageWidth = 512;
					axFvxImage1.ImageHeight = 480;
				}
				axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
			}

		}

		//画像の保存
		private void btnSaveImage_Click(object sender, EventArgs e)
		{
			saveFileDialog1.InitialDirectory = m_strDirectory;
			saveFileDialog1.Filter = "bitmap file(*.bmp)|*.bmp";
			saveFileDialog1.DefaultExt = "*.bmp";
			saveFileDialog1.ShowDialog();

			if (saveFileDialog1.FileName.Length > 0)
			{
				//FileAccessコンポーネントを使って
				//Imageをファイルに保存します｡
				axFvxFileAccess1.FilePath = saveFileDialog1.FileName;
				try
				{
					axFvxFileAccess1.SaveBitmap(axFvxImage1.GetOcx());
				}
				catch (Exception)
				{
					int IntErr = 0;
					axFvxFileAccess1.GetErrReport(out IntErr);
					MessageBox.Show("保存に失敗しました。　ErrCode= " + IntErr.ToString());
					return;
				}
			}

		}

		//処理範囲設定
		private void btnExeRegion_Click(object sender, EventArgs e)
		{
			int IntOK = 0;
			double dblTemp = 0.0;

			axFvxROI1.SetRectangle(m_ExeRegion);

			// FvxROIを使って処理範囲選択ダイアログを出します｡
			axFvxROI1.UsePoint = false;
			axFvxROI1.MakeROIDialog(axFvxImage1.GetOcx(), out IntOK);

			if ((IntOK == 1) && (axFvxROI1.CheckRectangle == 1))
			{
				axFvxROI1.GetRectangle(m_ExeRegion);
				// 始点と終点が逆になっていないかチェックします｡
				// FvxROIは矩形データで、左上・右下の座標位置(xs > xe, ys > ye)が
				// 逆になっていた場合でも、入れ替えずそのまま結果を返しますので、注意してください
				if (m_ExeRegion.xe < m_ExeRegion.xs)
				{
					dblTemp = m_ExeRegion.xs;
					m_ExeRegion.xs = m_ExeRegion.xe;
					m_ExeRegion.xe = dblTemp;
				}
				if (m_ExeRegion.ye < m_ExeRegion.ys)
				{
					dblTemp = m_ExeRegion.ys;
					m_ExeRegion.ys = m_ExeRegion.ye;
					m_ExeRegion.ye = dblTemp;
				}
			}

		}

		//表示クリア
		private void btnClear_Click(object sender, EventArgs e)
		{
			axFvxDisplayEx1.ClearOverlay(-1);
			txtResult.Text = "";
		}

		//実行
		private void btnExecute_Click(object sender, EventArgs e)
		{
			FvHoughResultGroup pHoughResultGroup = new FvHoughResultGroup();
			FvHoughResultGroup pRefineLineResult = new FvHoughResultGroup();
			double dblTime;
			int intCount;

			axFvxDisplayEx1.ClearOverlay(-1);

			//各パラメータの適用
			ApplyChanges();

			//時間計測開始
			axFvxPerformanceCounter1.StartCount();

			try
			{
				//ハフ平面の初期化
				axFvxHoughLine1.Init(axFvxImage1.GetOcx());

				//直線ハフ変換検出実行
				axFvxHoughLine1.Execute(axFvxImage1.GetOcx(), out intCount);

				if (chkRefineLine.Checked == true)
				{	//最小二乗法で直線を再計算
					axFvxHoughLine1.GetResult(-1, pHoughResultGroup);
					axFvxHoughLine1.RefineLine(pHoughResultGroup, m_intErrR, m_intErrQ, 0, pRefineLineResult);
				}
			}
			catch (System.Runtime.InteropServices.COMException)
			{
				int intErr;
				axFvxHoughLine1.GetErrReport(out intErr);
				MessageBox.Show(GetHoughLineErrMesg(intErr), "ハフ変換エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}
			//時間計測終了
			axFvxPerformanceCounter1.StopCount(out dblTime);

			//結果表示
			txtResult.Text = string.Format("処理時間：{0:00}ms\r\n\r\n --直線係数-- \r\n   a,    b,    c\r\n",dblTime); 
						
			axFvxDisplayEx1.PenColor = Color.Red;
			if(chkRefineLine.Checked == false)
			{
				axFvxHoughLine1.GetResult(-1, pHoughResultGroup);
				axFvxDisplayEx1.DrawData(0,pHoughResultGroup);

				for(int i = 0; i <= pHoughResultGroup.Count - 1; i++)
					txtResult.Text = txtResult.Text + string.Format("{0:0.00} , {1:0.00} , {2:0.00}\r\n",pHoughResultGroup.Item2(i).a,pHoughResultGroup.Item2(i).b,pHoughResultGroup.Item2(i).c);
			}
			else{
				axFvxDisplayEx1.DrawData(0,pRefineLineResult);

				for (int i = 0; i <= pRefineLineResult.Count - 1; i++)
					txtResult.Text = txtResult.Text + string.Format("{0:0.00} , {1:0.00} , {2:0.00}\r\n", pRefineLineResult.Item2(i).a, pRefineLineResult.Item2(i).b, pRefineLineResult.Item2(i).c);

			}			
		}

		private string GetHoughLineErrMesg(int intErr)
		{
			string Str;

			switch(intErr)
			{
				case 0:
					Str = "エラー無し";
					break;
				case 9605:
					Str = "パラメータが違います";
					break;
				case 9606:
					Str = "引数の方が違います";
					break;
				case 9609:
					Str = "パラメータにNULL値が指定されていました";
					break;
				case 9610:
					Str = "メモリの確保に失敗しました";
					break;
				case 9611:
					Str = "ハフ平面が生成できません";
					break;
				case 9620:
					Str = "ハフ平面が初期化できません";
					break;
				case 9621:
					Str = "エッジ配列が取得できません";
					break;
				case 9622:
					Str = "投票できません";
					break;
				case 9630:
					Str = "直線検出不可";
					break;
				case 9640:
					Str = "有効な結果がありません";
					break;
				default:
					Str = "不明なエラー";
					break;
			}
			return(Str);
		}
		
		//パラメータ設定
		private void ApplyChanges()
		{
			int intTemp, intStQ, intEdQ;

			//直線検出本数
			if (int.TryParse(txtInpNum.Text,out intTemp) == false || intTemp < 1)
			{
                MessageBox.Show("直線の本数は１以上の数を入力して下さい","パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
				return;
			}
			axFvxHoughLine1.InpNum = intTemp;

			//エッジ閾値
			if(int.TryParse(txtEdgeThreshold.Text,out intTemp) == false || (intTemp != -1) && (intTemp < 1) ||( intTemp > 255))
			{		
				MessageBox.Show("エッジ閾値は -1 or 1～255 の範囲で入力して下さい","パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
				return;
			}
			axFvxHoughLine1.EdgeThres = intTemp;

			//似たような直線を検出しないようにする為のパラメータ（距離）
			if(int.TryParse(txtRgnR.Text, out intTemp) == false || intTemp < 0)
			{
				MessageBox.Show("RgnRは 0 以上の数を入力して下さい","パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
				return;
			}
			axFvxHoughLine1.RgnR = intTemp;

			//似たような直線を検出しないようにする為のパラメータ（傾き）
			if(int.TryParse(txtRgnQ.Text,out intTemp) == false || intTemp < 0 || intTemp > 179)
			{
				MessageBox.Show("RgnQは 0～179 の範囲で入力して下さい","パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
				return;
			}
			axFvxHoughLine1.RgnQ = intTemp;

			//ハフ空間における投票の角度の範囲の片幅
			if(int.TryParse(txtVotWid.Text,out intTemp) == false || intTemp < 0 || intTemp > 179)
			{
				MessageBox.Show("辺幅は 0～179 の範囲で入力して下さい","パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
				return;
			}
			axFvxHoughLine1.VotWid = intTemp;

			//検出したい直線の傾きの範囲
			//-360 ≦ StQ ≦ 360,　 0 ≦ EdQ ≦ 360
			//StQ ≦ EdQ, 　　　   EdQ - StQ ≦ 360
			if(int.TryParse(txtStQ.Text, out intStQ) == false || intStQ < -360 || intStQ > 360)
			{
				MessageBox.Show("開始角度は -360°～ 360°の範囲で入力して下さい","パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
				return;
			}
			else if (int.TryParse(txtEdQ.Text,out intEdQ) == false || intEdQ < 0 || intEdQ > 360)
			{
				MessageBox.Show("終了角度は 0°～ 360°の範囲で入力して下さい ","パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
				return;
			}
			else if(intStQ > intEdQ)
			{
				MessageBox.Show("(開始角度)≦(終了角度)となるように入力して下さい","パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
				return;
			}
			else if((intEdQ - intStQ) > 360)
			{
				MessageBox.Show("(終了角度)≦(開始角度) ≦ 360 となるように入力して下さい","パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
				return;
			}
			axFvxHoughLine1.EdQ = intEdQ;
			axFvxHoughLine1.StQ = intStQ;

			//処理範囲
			axFvxHoughLine1.SetInspectArea(m_ExeRegion);
			axFvxDisplayEx1.PenColor = Color.Blue;
			axFvxDisplayEx1.DrawData(0, m_ExeRegion);

			if(chkRefineLine.Checked == true)
			{ //エッジ位置と直線の許容範囲
				if(int.TryParse(txtErrR.Text,out intTemp) == false || intTemp < 1)
				{
					MessageBox.Show("ErrRは１以上の数を入力して下さい","パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
					return;
				}
				m_intErrR = intTemp;

				if(int.TryParse(txtErrQ.Text,out intTemp) == false || intTemp < 1)
				{
					MessageBox.Show("ErrQは１以上の数を入力して下さい","パラメータエラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
					return;
				}
				m_intErrQ = intTemp; 
			}
		}

		//数値/Delete/BackSpace以外入力できないよう制御
		private void txtInpNum_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
				e.Handled = true;
		}

		private void txtEdgeThreshold_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '-' && e.KeyChar != '\b')
				e.Handled = true;
		}

		private void txtRgnR_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
				e.Handled = true;
		}

		private void txtRgnQ_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
				e.Handled = true;
		}

		private void txtVotWid_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
				e.Handled = true;
		}

		private void txtStQ_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '-' && e.KeyChar != '\b')
				e.Handled = true;
		}

		private void txtEdQ_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
				e.Handled = true;
		}

		private void txtErrR_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
				e.Handled = true;
		}

		private void txtErrQ_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
				e.Handled = true;
		}

		private void chkRefineLine_CheckedChanged(object sender, EventArgs e)
		{
			if (chkRefineLine.Checked == true)
			{
				txtErrQ.Enabled = true;
				txtErrR.Enabled = true;
			}
			else
			{
				txtErrQ.Enabled = false;
				txtErrR.Enabled = false;
			}
		}
	}
}